local libmg = LibStub("LibMouseGestures-1.0");
local doodle = libmg:New(UIParent);
local colors = {
	{1,0,0,1},
	{0,1,0,1},
	{0,0,1,1},
	{1,1,0,1},
	{1,0,1,1},
	{0,1,1,1}
}
local color = colors[1];
local colorx = 1;
local nodoodling = true;
local bases = { '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z' };
local otherdoodles = {};

function rebase(number, from, to)
	local num = tonumber(number, from) or 0;
	local y = to - 1;
	local retval = "";
	local rem = num;
	while ( num > y ) do
		local x = math.floor(num/to);
		rem = num - (x*to);
		num = x;
		retval = (bases[rem+1] or 0) .. (retval or 0);
	end
	retval = (bases[num+1] or 0) .. ( retval or 0);
	return retval;
end


function Doodlepad(chan, notMine)
    local frame = CreateFrame("Frame", nil, UIParent);
    frame:SetBackdrop({bgFile = "Interface/GLUES/loading",
                                            edgeFile = "Interface/GLUES/Common/TextPanel-Border",
                                            edgeSize = 16,
                                            insets = { left = 4, right = 4, top = 4, bottom = 4 }});
    frame:SetBackdropColor(1,1,1,0.4);
    frame:SetWidth(480);
    frame:SetHeight(320);
    frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	frame:RegisterForDrag("LeftButton");
	frame:SetScript("OnDragStart",function()  frame:StartMoving(); end);
	frame:SetScript("OnDragStop", function()  frame:StopMovingOrSizing(); end);
	frame:SetMovable(true);
	frame:EnableMouse(true);
	local subFrame = CreateFrame("Frame");
	subFrame:SetParent(frame);
	subFrame:SetPoint("TOPLEFT", 5, -28);
	subFrame:SetPoint("BOTTOMRIGHT", -5, 5);
	local canvas = subFrame:CreateTexture("ARTWORK");
	canvas:SetTexture(1,1,1,0.85);
	canvas:SetAllPoints(subFrame);


    -- close button
    local close = CreateFrame("button", nil, frame, "UIPanelCloseButton");
    close:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0);
    close:Show();

	local recorder = libmg:New(subFrame);
	recorder.doodles, recorder.cdoodle, recorder.texs, recorder.free = {},{},{},{};
	if ( type(notMine) ~= "string" ) then
		recorder:StartCapture({
			startButton		= "LeftButtonDown",
			stopButton		= "LeftButtonUp",
			cancelButton    = "RightButtonUp",
			startFunc 		= Doodle_Color,
			updateFunc		= Doodle_Update,
			stopFunc 		= Doodle_Stop,
			cancelFunc		= Doodle_Cancel,
		});
		chan = string.lower(chan or "");
		if ( chan == "raid" or chan == "party" or chan == "guild" ) then
			recorder.chan = chan;
			recorder.uid = ("%08x%08x"):format(math.random(time()), math.random(GetTime()));
			print("Broadcasting to " .. chan);
		end
	else
		recorder.uid = notMine;
		recorder.drawLayer:Raise();
		recorder.drawLayer:Show();
		recorder.drawLayer:EnableMouse(false);
		frame:Hide();
	end
	return frame, recorder;
end




function Doodle_DrawLine(rec, T, sx, sy, ex, ey, col)
	local color = col or color;
    sy = -sy;
    ey = -ey;
	local C = rec.drawLayer;
    w = 72;
    local dx,dy = ex - sx, ey - sy;
    local cx,cy = (sx + ex) / 2, (sy + ey) / 2;
    if (dx < 0) then
        dx,dy = -dx,-dy;
    end
    local Z = (256/255) / 2;
    local l = sqrt((dx * dx) + (dy * dy));
    local s,c = -dy / l, dx / l;
    local sc = s * c;
    local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
    if (dy >= 0) then
        Bwid = ((l * c) - (w * s)) * Z;
        Bhgt = ((w * c) - (l * s)) * Z;
        BLx, BLy, BRy = (w / l) * sc, s * s, (l / w) * sc;
        BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx;
        TRy = BRx;
    else
        Bwid = ((l * c) + (w * s)) * Z;
        Bhgt = ((w * c) + (l * s)) * Z;
        BLx, BLy, BRx = s * s, -(l / w) * sc, 1 + (w / l) * sc;
        BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy;
        TRx = TLy;
    end
    T:SetDrawLayer(layer or "OVERLAY")
    T:SetVertexColor(color[1],color[2],color[3],color[4]);
    T:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy);
    T:SetPoint("TOPLEFT",   C, "TOPLEFT", cx - Bwid, cy + Bhgt);
    T:SetWidth(Bwid*2);
    T:SetHeight(Bhgt*2);
    T:Show()
end

function Doodlepad_Incoming(_,_,prefix,msg,chan,sender)
	if ( prefix == "DPD" ) then
		local uid, col, data = msg:sub(1,16), msg:sub(17,17), msg:sub(18);
		local Color = colors[tonumber(col) or 1] or colors[1];
		local found = false;
		local rec = nil;
		for n = 1, #otherdoodles do
			if ( otherdoodles[n].uid == uid ) then
				found = true;
				rec = otherdoodles[n].recorder;
				break;
			end
		end
		if ( not found ) then
			local frame, recorder = Doodlepad(chan, uid);
			table.insert(otherdoodles, { frame = frame, recorder = recorder, uid = uid} );
			rec = recorder;
			print(string.format("%s is broadcasting a doodle. |HDPD:%s|h|cFFFFFF00[Click here]|r|h to view it",sender, uid));
		end
		if ( rec ) then
			if ( #rec.cdoodle > 0 ) then
				table.insert(rec.doodles, rec.cdoodle);
				rec.cdoodle = {};
			end
			for n = 1, math.floor(data:len()/4) do
				local p = ((n-1)*4)+1;
				local x, y = rebase(data:sub(p, p+1), 32, 10), rebase(data:sub(p+2, p+3), 32, 10);
				table.insert(rec.cdoodle, {x,y});
				Doodle_Draw(rec, false, Color);
			end
		end
	end
end

function Doodle_ParseLink(link, text, button, chatframe)
	local linktype, id = strsplit(":", link)
	if linktype == "DPD" then
		local uid = id;
		for n = 1, #otherdoodles do
			if ( otherdoodles[n].uid == uid ) then
				otherdoodles[n].frame:Show();
				break;
			end
		end
	else
		doodle.itemref(link, text, button, chatframe);
	end
end


function Doodle_Update(rec,a,b,c,d)
	if ( #rec.cdoodle > 0 ) then
		local l = rec.cdoodle[#rec.cdoodle];
		if ( math.floor(l[1]) ~= math.floor(c) or math.floor(l[2]) ~= math.floor(d) ) then
			if ( abs( l[1] - c) < 1 and abs(l[2] - d) < 1 ) then -- lines that don't go anywhere makes me a sad panda.
				return;
			end
			local dist = math.sqrt( math.pow(l[1] - c, 2) + math.pow(l[2] - d, 2));
			if ( dist > 5 ) then
				table.insert(rec.cdoodle, {c,d});
			end
		end
	else
		table.insert(rec.cdoodle, {c,d});
	end
	Doodle_Draw(rec);
end

function Doodle_Color()
	if ( IsShiftKeyDown() ) then
		colorx = colorx + 1;
		if ( colorx > #colors ) then colorx = 1; end
		color = colors[colorx];
	end
end

function Doodle_Stop(rec)
	table.insert(rec.doodles, rec.cdoodle);
	rec:StartCapture({
		startButton		= "LeftButtonDown",
		stopButton		= "LeftButtonUp",
		cancelButton    = "RightButtonUp",
		startFunc 		= Doodle_Color,
		updateFunc		= Doodle_Update,
		stopFunc 		= Doodle_Stop,
		cancelFunc		= Doodle_Cancel,
	});
	local s = 0;
	for b = 1, #rec.cdoodle do
		s = s + 4;
	end
	if ( rec.chan ) then
		local x = 0;
		local c = "";
		for n = 1, #rec.cdoodle do
			x = x + 1;
			c = c .. ("%2s%2s"):format(rebase(math.floor(rec.cdoodle[n][1]), 10, 32), rebase(math.floor(rec.cdoodle[n][2]), 10, 32));
			if ( x > 50 ) then
				SendAddonMessage("DPD",rec.uid .. colorx .. c, rec.chan);
				c = "";
				x = 0;
			end
		end
		if ( c:len() > 0 ) then
			SendAddonMessage("DPD",rec.uid .. colorx .. c, rec.chan);
		end
	end
	rec.cdoodle = {};
end

function Doodle_Cancel(rec)
	Doodle_Draw(rec, true);
	if ( #rec.doodles > 0 ) then
		rec:StartCapture({
			startButton		= "LeftButtonDown",
			stopButton		= "LeftButtonUp",
			cancelButton    = "RightButtonUp",
			startFunc 		= Doodle_Color,
			updateFunc		= Doodle_Update,
			stopFunc 		= Doodle_Stop,
			cancelFunc		= Doodle_Cancel,
		});
	else
		nodoodling = true;
		print("Exited doodle mode.");
	end
	rec.doodles = {};
	rec.cdoodle = {};
end

function Doodle_Draw(rec, wipe, col)
	if ( wipe == true ) then
		for n = 1, #rec.texs do
			rec.texs[n]:Hide();
			rec.texs[n]:SetTexCoord(0,1,0,1);
			table.insert(rec.free, rec.texs[n]);
		end
		texs = {};
	else
		for n = 1, #rec.cdoodle do
			if ( not rec.cdoodle[n][3] and rec.cdoodle[n-1] ) then
				local T = table.remove(rec.free) or rec.drawLayer:CreateTexture();
				T:SetTexture([[Interface\AddOns\Doodle\line]]);
				rec.cdoodle[n][3] = T;
				table.insert(rec.texs, T);
				Doodle_DrawLine(rec, T, rec.cdoodle[n-1][1], rec.cdoodle[n-1][2], rec.cdoodle[n][1], rec.cdoodle[n][2], col);
				T:Show();
			end
		end
	end
end


do
doodle.drawLayer:Hide();
    _G.SLASH_DOODLEFULL1 = '/doodle';
	_G.SLASH_DOODLE1 = '/doodlepad';
	SlashCmdList['DOODLE'] = Doodlepad;
	SlashCmdList['DOODLEFULL'] = function()
		if ( doodle.drawLayer:IsVisible() ) then
			doodle.drawLayer:Hide()
		else
			if ( nodoodling == true ) then
				doodle.doodles, doodle.cdoodle, doodle.texs, doodle.free = {},{},{},{};
				print("You are now in doodle mode. Use right click to wipe the screen and double right click to exit doodle mode.");
				nodoodling = false;
			doodle:StartCapture({
				startButton		= "LeftButtonDown",
				stopButton		= "LeftButtonUp",
				cancelButton    = "RightButtonUp",
				startFunc 		= Doodle_Color,
				updateFunc		= Doodle_Update,
				stopFunc 		= Doodle_Stop,
				cancelFunc		= Doodle_Cancel,
			});
			end
			doodle.drawLayer:Show();
		end
	end
	local cf = CreateFrame("Frame");
	cf:SetParent(UIParent);
	cf:SetPoint("TOPLEFT");
	cf:SetWidth(1);
	cf:SetHeight(1);
	cf:RegisterEvent("CHAT_MSG_ADDON");
	cf:SetScript("OnEvent", Doodlepad_Incoming);
	doodle.itemref = _G.SetItemRef;
	_G.SetItemRef = Doodle_ParseLink;
end
